@@ -23,12 +23,15 @@ public:
 
   auto*               network_config()     { return m_network_config.get(); }
 
+  // TODO: Remove.
+  auto*               network_manager()    { return runtime::network_manager(); }
+  auto*               socket_manager()     { return runtime::socket_manager(); }
+
   auto*               chunk_manager()      { return m_chunk_manager.get(); }
   auto*               connection_manager() { return m_connection_manager.get(); }
   auto*               download_manager()   { return m_download_manager.get(); }
   auto*               file_manager()       { return m_file_manager.get(); }
   auto*               handshake_manager()  { return m_handshake_manager.get(); }
-  auto*               network_manager()    { return m_network_manager.get(); }
   auto*               resource_manager()   { return m_resource_manager.get(); }
 
   auto*               client_list()        { return m_client_list.get(); }
@@ -46,20 +49,19 @@ public:
 private:
   std::unique_ptr<net::NetworkConfig>  m_network_config;
 
-  std::unique_ptr<ChunkManager>        m_chunk_manager;
-  std::unique_ptr<ConnectionManager>   m_connection_manager;
-  std::unique_ptr<DownloadManager>     m_download_manager;
-  std::unique_ptr<FileManager>         m_file_manager;
-  std::unique_ptr<HandshakeManager>    m_handshake_manager;
-  std::unique_ptr<net::NetworkManager> m_network_manager;
-  std::unique_ptr<ResourceManager>     m_resource_manager;
+  std::unique_ptr<ChunkManager>      m_chunk_manager;
+  std::unique_ptr<ConnectionManager> m_connection_manager;
+  std::unique_ptr<DownloadManager>   m_download_manager;
+  std::unique_ptr<FileManager>       m_file_manager;
+  std::unique_ptr<HandshakeManager>  m_handshake_manager;
+  std::unique_ptr<ResourceManager>   m_resource_manager;
 
-  std::unique_ptr<ClientList>             m_client_list;
+  std::unique_ptr<ClientList>        m_client_list;
 
-  EncodingList        m_encodingList;
+  EncodingList          m_encodingList;
 
-  Throttle*           m_uploadThrottle;
-  Throttle*           m_downloadThrottle;
+  Throttle*             m_uploadThrottle;
+  Throttle*             m_downloadThrottle;
 
   unsigned int          m_ticks{0};
   utils::SchedulerEntry m_task_tick;
