@@ -3,13 +3,13 @@
 
 #include <cinttypes>
 #include <functional>
+#include <memory>
 
-#include "socket_base.h"
-#include "socket_fd.h"
+#include "torrent/event.h"
 
 namespace torrent {
 
-class Listen : public SocketBase {
+class Listen : public Event {
 public:
   ~Listen() override { close(); }
 
@@ -24,9 +24,7 @@ public:
 
   void                close();
 
-  bool                is_open() const { return get_fd().is_valid(); }
-
-  uint16_t            port() const { return m_port; }
+  uint16_t            port() const    { return m_port; }
 
   auto&               slot_accepted() { return m_slot_accepted; }
 
@@ -35,11 +33,12 @@ public:
   void                event_error() override;
 
 private:
+  using slot_accept_type = std::function<void(std::unique_ptr<Handshake>& handshake, int, const sockaddr*)>;
+
   void                open_done(int fd, uint16_t port, int backlog);
 
   uint16_t            m_port{0};
-
-  std::function<void(int, const sockaddr*)> m_slot_accepted;
+  slot_accept_type    m_slot_accepted;
 };
 
 } // namespace torrent
