@@ -11,7 +11,6 @@
 #include "torrent/download/choke_queue.h"
 #include "torrent/download_info.h"
 #include "torrent/net/socket_address.h"
-#include "torrent/net/network_manager.h"
 #include "torrent/peer/connection_list.h"
 #include "torrent/peer/peer_info.h"
 #include "torrent/utils/log.h"
@@ -295,7 +294,7 @@ PeerConnection<type>::read_message() {
     if (!m_down->can_read_port_body())
       break;
 
-    runtime::network_manager()->dht_add_peer_node(m_peerInfo->socket_address(), m_down->buffer()->read_16());
+    runtime::dht_add_peer_node(m_peerInfo->socket_address(), m_down->buffer()->read_16());
     return true;
 
   case ProtocolBase::EXTENSION_PROTOCOL:
@@ -449,7 +448,7 @@ PeerConnection<type>::event_read() {
 
   } catch (const base_error& e) {
     std::stringstream s;
-    s << "Connection read fd(" << get_fd().get_fd() << ',' << m_down->get_state() << ',' << m_down->last_command() << ") \"" << e.what() << '"';
+    s << "Connection read fd(" << m_fileDesc << ',' << m_down->get_state() << ',' << m_down->last_command() << ") \"" << e.what() << '"';
     s << " '" << utils::uri_escape_html(reinterpret_cast<char*>(m_down->buffer()->begin()), reinterpret_cast<char*>(m_down->buffer()->position())) << "'";
 
     throw internal_error(s.str());
@@ -630,7 +629,7 @@ PeerConnection<type>::event_write() {
 
   } catch (const base_error& e) {
     std::stringstream s;
-    s << "Connection write fd(" << get_fd().get_fd() << ',' << m_up->get_state() << ',' << m_up->last_command() << ") \"" << e.what() << '"';
+    s << "Connection write fd(" << m_fileDesc << ',' << m_up->get_state() << ',' << m_up->last_command() << ") \"" << e.what() << '"';
 
     throw internal_error(s.str());
   }
