@@ -26,9 +26,19 @@
 #define LT_LOG_EVENT(log_fmt, ...)                                      \
   lt_log_print(LOG_CONNECTION_FD, "kqueue->%i : %s : " log_fmt, event->file_descriptor(), event->type_name(), __VA_ARGS__);
 
+#if 1
+
+#define LT_LOG_DEBUG(log_fmt, ...)
 #define LT_LOG_DEBUG_IDENT(log_fmt, ...)
-// #define LT_LOG_DEBUG_IDENT(log_fmt, ...)                                \
-//   lt_log_print(LOG_CONNECTION_FD, "kqueue->%u: " log_fmt, static_cast<unsigned int>(itr->ident), __VA_ARGS__);
+
+#else
+
+#define LT_LOG_DEBUG(log_fmt, ...)                                  \
+  lt_log_print(LOG_CONNECTION_FD, "kqueue: " log_fmt, __VA_ARGS__);
+#define LT_LOG_DEBUG_IDENT(log_fmt, ...)                                \
+  lt_log_print(LOG_CONNECTION_FD, "kqueue->%u : " log_fmt, static_cast<unsigned int>(itr->ident), __VA_ARGS__);
+
+#endif
 
 namespace torrent::net {
 
@@ -98,7 +108,7 @@ PollInternal::flush() {
   if (m_changed_events == 0)
     return;
 
-  LT_LOG("flushing events : changed:%u", m_changed_events);
+  LT_LOG_DEBUG("flushing events : changed:%u", m_changed_events);
 
   if (::kevent(m_fd, m_changes.get(), m_changed_events, nullptr, 0, nullptr) == -1)
     throw internal_error("PollInternal::flush() error: " + std::string(std::strerror(errno)));
@@ -192,40 +202,40 @@ Poll::poll(int timeout_usec) {
 
 unsigned int
 Poll::process() {
-  unsigned int count = 0;
+  unsigned int count{};
+
+  m_processing = true;
+  m_closed_events.clear();
 
   for (struct kevent *itr = m_internal->m_events.get(), *last = m_internal->m_events.get() + m_internal->m_waiting_events; itr != last; ++itr) {
     if (utils::Thread::self()->callbacks_should_interrupt_polling())
       utils::Thread::self()->process_callbacks(true);
 
     auto* poll_event = static_cast<PollEvent*>(itr->udata);
-    auto* event      = poll_event->event;
-
-    if (event == nullptr) {
-      // TODO: This should fail.
-      LT_LOG_DEBUG_IDENT("event is null, skipping : udata:%p", itr->udata);
-      continue;
-    }
 
     if ((itr->flags & EV_ERROR)) {
+      count++;
+
+      if (poll_event->event == nullptr)
+        continue;
+
       if (!(poll_event->mask & PollInternal::flag_error))
         throw internal_error("Poll::process() received error event for event not in error: " + poll_event->event->print_name_fd_str());
 
-      auto event_info = event->print_name_fd_str();
+      auto event_info = poll_event->event->print_name_fd_str();
 
-      event->event_error();
+      poll_event->event->event_error();
 
       if (poll_event->mask != 0)
         throw internal_error("Poll::process() event_error called but event mask not cleared: " + event_info);
 
       // We assume that the event gets closed if we get an error.
-      count++;
       continue;
     }
 
     if (itr->filter == EVFILT_READ && (poll_event->mask & PollInternal::flag_read)) {
       count++;
-      event->event_read();
+      poll_event->event->event_read();
     }
     else if (itr->filter == EVFILT_READ) {
       LT_LOG_DEBUG_IDENT("spurious read event, skipping", 0);
@@ -233,7 +243,7 @@ Poll::process() {
 
     if (itr->filter == EVFILT_WRITE && (poll_event->mask & PollInternal::flag_write)) {
       count++;
-      event->event_write();
+      poll_event->event->event_write();
     }
     else if (itr->filter == EVFILT_WRITE) {
       LT_LOG_DEBUG_IDENT("spurious write event, skipping", 0);
@@ -241,7 +251,11 @@ Poll::process() {
 
   }
 
+  m_closed_events.clear();
+  m_processing = false;
+
   m_internal->m_waiting_events = 0;
+
   return count;
 }
 
@@ -288,8 +302,11 @@ Poll::close(Event* event) {
 
   m_internal->flush();
 
-  poll_event->event = nullptr;
-  event->m_poll_event.reset();
+  if (m_processing)
+    m_closed_events.push_back(event->m_poll_event);
+
+  poll_event->event   = nullptr;
+  event->m_poll_event = nullptr;
 }
 
 bool
@@ -311,10 +328,8 @@ void
 Poll::insert_read(Event* event) {
   auto event_mask = m_internal->event_mask(event);
 
-  if (event_mask & PollInternal::flag_read) {
-    LT_LOG_EVENT("insert read: already in read", 0);
+  if (event_mask & PollInternal::flag_read)
     return;
-  }
 
   LT_LOG_EVENT("insert read", 0);
 
@@ -326,10 +341,8 @@ void
 Poll::insert_write(Event* event) {
   auto event_mask = m_internal->event_mask(event);
 
-  if (event_mask & PollInternal::flag_write) {
-    LT_LOG_EVENT("insert write: already in write", 0);
+  if (event_mask & PollInternal::flag_write)
     return;
-  }
 
   LT_LOG_EVENT("insert write", 0);
 
@@ -339,17 +352,23 @@ Poll::insert_write(Event* event) {
 
 void
 Poll::insert_error(Event* event) {
+  auto event_mask = m_internal->event_mask(event);
+
+  if (event_mask & PollInternal::flag_error)
+    return;
+
   LT_LOG_EVENT("insert error", 0);
+
+  m_internal->set_event_mask(event, event_mask | PollInternal::flag_error);
+  // Kqueue always monitors errors, no need to insert.
 }
 
 void
 Poll::remove_read(Event* event) {
   auto event_mask = m_internal->event_mask(event);
 
-  if (!(event_mask & PollInternal::flag_read)) {
-    LT_LOG_EVENT("remove read: not in read", 0);
+  if (!(event_mask & PollInternal::flag_read))
     return;
-  }
 
   LT_LOG_EVENT("remove read", 0);
 
@@ -361,10 +380,8 @@ void
 Poll::remove_write(Event* event) {
   auto event_mask = m_internal->event_mask(event);
 
-  if (!(event_mask & PollInternal::flag_write)) {
-    LT_LOG_EVENT("remove write: not in write", 0);
+  if (!(event_mask & PollInternal::flag_write))
     return;
-  }
 
   LT_LOG_EVENT("remove write", 0);
 
@@ -374,7 +391,15 @@ Poll::remove_write(Event* event) {
 
 void
 Poll::remove_error(Event* event) {
+  auto event_mask = m_internal->event_mask(event);
+
+  if (!(event_mask & PollInternal::flag_error))
+    return;
+
   LT_LOG_EVENT("remove error", 0);
+
+  m_internal->set_event_mask(event, event_mask & ~PollInternal::flag_error);
+  // Kqueue always monitors errors, no need to remove.
 }
 
 void
