@@ -10,9 +10,9 @@
 #include "torrent/connection_manager.h"
 #include "torrent/net/fd.h"
 #include "torrent/net/network_config.h"
-#include "torrent/net/network_manager.h"
 #include "torrent/net/resolver.h"
 #include "torrent/net/socket_address.h"
+#include "torrent/runtime/socket_manager.h"
 #include "torrent/utils/log.h"
 #include "torrent/utils/option_strings.h"
 
@@ -37,7 +37,7 @@ TrackerUdp::~TrackerUdp() {
 
 bool
 TrackerUdp::is_busy() const {
-  return m_fileDesc != -1;
+  return is_open();
 }
 
 void
@@ -133,13 +133,15 @@ TrackerUdp::close_directly() {
   m_read_buffer.reset();
   m_write_buffer.reset();
 
-  if (!get_fd().is_valid())
+  if (!is_open())
     return;
 
-  this_thread::event_remove_and_close(this);
+  runtime::socket_manager()->close_event_or_throw(this, [this]() {
+      this_thread::event_remove_and_close(this);
 
-  get_fd().close();
-  get_fd().clear();
+      fd_close(file_descriptor());
+      set_file_descriptor(-1);
+    });
 }
 
 tracker_enum
@@ -213,7 +215,7 @@ TrackerUdp::start_announce() {
   if (!m_sending_announce)
     throw internal_error("TrackerUdp::start_announce() called but m_sending_announce is false.");
 
-  if (m_fileDesc != -1)
+  if (is_open())
     throw internal_error("TrackerUdp::start_announce() called but m_fileDesc is already open.");
 
   m_sending_announce = false;
@@ -257,9 +259,38 @@ TrackerUdp::start_announce() {
   LT_LOG("starting announce : connect_address:%s bind_address:%s",
          sa_pretty_str(m_current_address).c_str(), sa_pretty_str(bind_address.get()).c_str());
 
-  m_fileDesc = fd_open_family(fd_flag_datagram | fd_flag_nonblock, bind_address->sa_family);
+  auto open_fd = [this, &bind_address]() {
+      int fd = fd_open_family(fd_flag_datagram | fd_flag_nonblock, bind_address->sa_family);
 
-  if (m_fileDesc == -1) {
+      if (fd == -1)
+        return;
+
+      set_file_descriptor(fd);
+
+      // TODO: Add socket counting.
+      this_thread::event_open(this);
+      this_thread::event_insert_read(this);
+      this_thread::event_insert_write(this);
+      this_thread::event_insert_error(this);
+    };
+
+  auto cleanup_func = [this]() {
+      if (!is_open()) {
+        LT_LOG("failed to open UDP socket : open failed", 0);
+        return;
+      }
+
+      LT_LOG("failed to open UDP socket : socket manager triggered cleanup", 0);
+
+      this_thread::event_remove_and_close(this);
+
+      fd_close(file_descriptor());
+      set_file_descriptor(-1);
+    };
+
+  runtime::socket_manager()->open_event_or_cleanup(this, open_fd, cleanup_func);
+
+  if (!is_open()) {
     LT_LOG("could not open UDP socket: fd:%i error:'%s'", m_fileDesc, std::strerror(errno));
     return receive_failed("could not open UDP socket: " + std::string(std::strerror(errno)));
   }
@@ -277,12 +308,6 @@ TrackerUdp::start_announce() {
 
   prepare_connect_input();
 
-  // TODO: Add socket counting.
-  this_thread::event_open(this);
-  this_thread::event_insert_read(this);
-  this_thread::event_insert_write(this);
-  this_thread::event_insert_error(this);
-
   m_tries = udp_tries;
 
   this_thread::scheduler()->wait_for_ceil_seconds(&m_task_timeout, std::chrono::seconds(udp_timeout));
@@ -386,7 +411,7 @@ TrackerUdp::prepare_announce_input() {
 
   m_write_buffer->write_32(info().key);
   m_write_buffer->write_32(parameters.numwant);
-  m_write_buffer->write_16(runtime::network_manager()->listen_port_or_throw());
+  m_write_buffer->write_16(runtime::listen_port());
 
   if (m_write_buffer->size_end() != 98)
     throw internal_error("TrackerUdp::prepare_announce_input() ended up with the wrong size");
